/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.settings;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;
import java.util.UUID;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.user.common.callcontext.CallContextUserTestFactory;
import de.ozgcloud.user.common.callcontext.CurrentCallContextUserService;
import io.quarkus.hal.HalEntityWrapper;

class UserSettingsResourceAssemblerTest {

	@Spy
	@InjectMocks
	private UserSettingsResourceAssembler assembler;
	@Mock
	private CurrentCallContextUserService currentUserService;

	@DisplayName("To resource")
	@Nested
	class TestToResource {

		private static final String USER_MANAGER_URL = LoremIpsum.getInstance().getUrl();

		private static final String ID = UUID.randomUUID().toString();

		@Test
		void shouldAddSelfLink() {
			doReturn(false).when(assembler).isUserAuthorized();

			var res = toResource();

			assertThat(res.getLinks().get(UserSettingsResourceAssembler.REL_SELF))
					.isNotNull()
					.hasFieldOrPropertyWithValue("href", USER_MANAGER_URL + UserSettingsResource.SETTINGS_LINK_PATTERN.replace("{id}", ID));
		}

		@DisplayName("edit link")
		@Nested
		class TestEditLink {

			private static final String EDIT_LINK = USER_MANAGER_URL + UserSettingsResource.SETTINGS_LINK_PATTERN.replace("{id}", ID);

			@Test
			void shouldBeAddedIfUserHasVerwaltungUserRoleAndEmail() {
				when(currentUserService.hasRole(anyString())).thenReturn(true);
				when(currentUserService.getCurrentCallContextUser()).thenReturn(Optional.of(CallContextUserTestFactory.create()));

				var resource = toResource();

				assertThat(resource.getLinks().get(UserSettingsResourceAssembler.REL_EDIT))
						.isNotNull()
						.hasFieldOrPropertyWithValue("href", EDIT_LINK);
			}

			@Test
			void shouldNotBeAddedIfUserHasNoEmail() {
				doReturn(false).when(assembler).hasValidEmail();

				var resource = toResource();

				assertThat(resource.getLinks()).doesNotContainKey(UserSettingsResourceAssembler.REL_EDIT);
			}

			@Test
			void shouldNotBeAddedIfUserNotHasVerwaltungUserRole() {
				doReturn(true).when(assembler).hasValidEmail();
				doReturn(false).when(assembler).hasVerwaltungUserRole();

				var resource = toResource();

				assertThat(resource.getLinks()).doesNotContainKey(UserSettingsResourceAssembler.REL_EDIT);
			}
		}

		private HalEntityWrapper toResource() {
			return assembler.toResource(UserSettingsTestFactory.create(), ID, USER_MANAGER_URL);
		}
	}
}