/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.userprofile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserService;
import de.ozgcloud.user.UserTestFactory;
import io.grpc.stub.StreamObserver;
import lombok.SneakyThrows;

class UserProfileGrpcServiceTest {

	@Spy
	@InjectMocks
	private UserProfileGrpcService grpcService;
	@Mock
	private UserService userService;

	@Mock
	private UserProfileMapper mapper;

	@DisplayName("Get userprofile by id")
	@Nested
	class TestGetUserProfileById {

		@Mock
		private StreamObserver<GrpcGetUserProfileResponse> streamObserver;

		@Captor
		private ArgumentCaptor<GrpcGetUserProfileResponse> responseCaptor;

		private final User userProfile = UserTestFactory.create();
		private final String userId = UUID.randomUUID().toString();
		private final GrpcGetUserProfileRequest request = GrpcGetUserProfileRequest.newBuilder().setUserId(userId).build();
		private final GrpcUserProfile grpcUserProfile = GrpcUserProfileTestFactory.create();

		@BeforeEach
		void init() {
			when(userService.getById(anyString())).thenReturn(userProfile);
			when(mapper.mapTo(any())).thenReturn(grpcUserProfile);
		}

		@Test
		void shouldCallService() {
			callGetById();

			verify(userService).getById(userId);
		}

		@Test
		void shouldMapUserProfile() {
			callGetById();

			verify(mapper).mapTo(userProfile);
		}

		@Test
		void shouldCallOnNext() {
			callGetById();

			verify(streamObserver).onNext(any(GrpcGetUserProfileResponse.class));
		}

		@Test
		void shouldCallOnCompleted() {
			callGetById();

			verify(streamObserver).onCompleted();
		}

		@Test
		void shouldReturnUserProfile() {
			callGetById();

			verify(streamObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue().getUserProfile().getFirstName()).isEqualTo(UserTestFactory.FIRST_NAME);
			assertThat(responseCaptor.getValue().getUserProfile().getLastName()).isEqualTo(UserTestFactory.LAST_NAME);
		}

		@SneakyThrows
		private void callGetById() {
			grpcService.getById(request, streamObserver);
		}
	}

	@Nested
	class TestGetByExternalId {

		private static final String EXTERNAL_ID = "42";

		@Mock
		private StreamObserver<GrpcGetUserProfileResponse> streamObserver;

		@Captor
		private ArgumentCaptor<GrpcGetUserProfileResponse> responseCaptor;

		@Test
		void shouldCallService() {
			when(mapper.mapTo(any())).thenReturn(GrpcUserProfileTestFactory.create());
			grpcService.getByExternalId(GrpcGetUserProfileRequest.newBuilder().setUserId(EXTERNAL_ID).build(), streamObserver);

			verify(userService).findByExternalId(EXTERNAL_ID);
		}

		@Test
		void shouldEmitUserProfile() {
			when(mapper.mapTo(any())).thenReturn(GrpcUserProfileTestFactory.create());
			grpcService.getByExternalId(GrpcGetUserProfileRequest.newBuilder().setUserId(EXTERNAL_ID).build(), streamObserver);

			verify(streamObserver).onNext(any());
		}

		@Test
		void shouldCompleteResponse() {
			when(mapper.mapTo(any())).thenReturn(GrpcUserProfileTestFactory.create());
			grpcService.getByExternalId(GrpcGetUserProfileRequest.newBuilder().setUserId(EXTERNAL_ID).build(), streamObserver);

			verify(streamObserver).onCompleted();
		}
	}

	@Nested
	class TestGetAllByOrganisationsEinheitId {

		@Mock
		private StreamObserver<GrpcGetAllByOrganisationsEinheitIdResponse> streamObserver;

		private final Stream<User> users = Stream.of(UserTestFactory.create());
		private final GrpcGetAllByOrganisationsEinheitIdResponse response = GrpcGetAllByOrganisationsEinheitIdResponseTestFactory.create();

		@BeforeEach
		void mock() {
			when(userService.findAllUsersOfOrganisationsEinheit(UserTestFactory.ORGANISTATIONSEINHEITEN_ID)).thenReturn(users);
			doReturn(response).when(grpcService).buildGrpcGetAllByOrganisationsEinheitIdResponse(users);
		}

		@Test
		void shouldCallUserService() {
			callService();

			verify(userService).findAllUsersOfOrganisationsEinheit(UserTestFactory.ORGANISTATIONSEINHEITEN_ID);
		}

		@Test
		void shouldBuildGrpcGetAllByOrganisationsEinheitIdResponse() {
			callService();

			verify(grpcService).buildGrpcGetAllByOrganisationsEinheitIdResponse(users);
		}

		@Test
		void shouldCallOnNext() {
			callService();

			verify(streamObserver).onNext(response);
		}

		@Test
		void shouldCallOnCompleted() {
			callService();

			verify(streamObserver).onCompleted();
		}

		private void callService() {
			grpcService.getAllByOrganisationsEinheitId(GrpcGetAllByOrganisationsEinheitIdRequestTestFactory.create(), streamObserver);
		}
	}

	@Nested
	class TestBuildGrpcGetAllByOrganisationsEinheitIdResponse {

		private final User user = UserTestFactory.create();
		private final Stream<User> users = Stream.of(user);
		private GrpcUserProfile grpcUserProfile = GrpcUserProfileTestFactory.create();

		@BeforeEach
		void mock() {
			when(mapper.mapTo(user)).thenReturn(grpcUserProfile);
		}

		@Test
		void shouldMapUser() {
			callService();

			verify(mapper).mapTo(user);
		}

		@Test
		void shouldMappedUsersInResponse() {
			var response = callService();

			assertThat(response.getUserProfileList()).containsExactly(grpcUserProfile);
		}

		private GrpcGetAllByOrganisationsEinheitIdResponse callService() {
			return grpcService.buildGrpcGetAllByOrganisationsEinheitIdResponse(users);
		}
	}

	@Nested
	class TestBuildGrpcGetInactiveUserIdsResponse {

		@Test
		void shouldBuildResponse() {
			var response = grpcService.buildGrpcGetInactiveUserIdsResponse(Stream.of(UserTestFactory.ID_STR));

			assertThat(response.getUserProfileIdsList()).first().extracting(GrpcUserProfileId::getId).isEqualTo(UserTestFactory.ID_STR);
		}

	}

	@Nested
	class TestFindInactiveUserIds {

		@Mock
		private StreamObserver<GrpcFindInactiveUserIdsResponse> streamObserver;

		private GrpcFindInactiveUserIdsResponse grpcResponse = GrpcGetInactiveUserIdsResponseTestFactory.create();

		private final Stream<String> userIds = Stream.of(UserTestFactory.ID_STR);

		@BeforeEach
		void init() {
			when(userService.findAllInactiveUserIds()).thenReturn(userIds);
			doReturn(grpcResponse).when(grpcService).buildGrpcGetInactiveUserIdsResponse(userIds);
		}

		@Test
		void shouldCallService() {
			callService();

			verify(userService).findAllInactiveUserIds();
		}

		@Test
		void shouldBuildResponse() {
			callService();

			verify(grpcService).buildGrpcGetInactiveUserIdsResponse(userIds);
		}

		@Test
		void shouldCallOnNext() {
			callService();

			verify(streamObserver).onNext(grpcResponse);
		}

		@Test
		void shouldCompleteResponse() {
			callService();

			verify(streamObserver).onCompleted();
		}

		private void callService() {
			grpcService.findInactiveUserIds(GrpcFindInactiveUserIdsRequest.newBuilder().build(), streamObserver);
		}
	}

	@Nested
	class TestDeleteInactiveUser {

		@Mock
		private StreamObserver<GrpcDeleteInactiveUserResponse> streamObserver;

		@Test
		void shouldCallService() {
			callService();

			verify(userService).deleteInactive(GrpcDeleteInactiveUserRequestTestFactory.USER_ID);
		}

		@Test
		void shouldCallOnNext() {
			callService();

			verify(streamObserver).onNext(GrpcDeleteInactiveUserResponse.newBuilder().build());
		}

		@Test
		void shouldCompleteResponse() {
			callService();

			verify(streamObserver).onCompleted();
		}

		private void callService() {
			grpcService.deleteInactiveUser(GrpcDeleteInactiveUserRequestTestFactory.create(), streamObserver);
		}
	}
}